//////////////////////////////////////////////////////////////////////////////////////
//  
//  FMFatLoader.c - 24U FM FatLoader for loading native PPC fragment on Mac OS
//
//  Modified public version distributed as part of 24U FM Template 1.0
//  
//  Copyright 2002-2003 by 24U Software. All rights reserved.
//
//  Written by Tomas Zahradnicky
//
////////////////////////////////////////////////////////////////////////////////
//
//  24U Sample Code - Terms of use
//
//  You may incorporate this sample code into your products without
//  restriction, though the code has been provided "AS IS" and the
//  responsibility for its operation is 100% yours.
//
//  You are not however permitted to redistribute the source
//  as "24U Sample Code" after having made changes.
//
//  If you are going to redistribute the source, we require that
//  you make it clear in the source that the code was descended from
//  24U Sample Code, but that you've made changes.
//
//  If you incorporate this sample code into your own product which you
//  distribute, you must include the following statement in the copyright
//  notices of your documentation: "Portions of the software are based
//  on 24U Sample Code which is copyright 2002-2003 by 24U Software.
//  All rights reserved."
//
////////////////////////////////////////////////////////////////////////

//
//  This source file depends on private FMFatLoader library.
//  Use of this private library is strictly limited to building FileMaker Pro
//  plug-ins based on 24U FM Template. Any use of the FMFatLoader library
//  in any other kind of project is strictly prohibited.
//
////////////////////////////////////////////////////////////////////////////////

//
//  Use this file for the 68K part of the SuperFAT version of your plug-in.
//  Keep in mind that the targets for the 68K and PPC Classic parts must be
//  named "68K" and "PPC Classic" respectively otherwise the FatLoader will
//  not get called properly.
//
////////////////////////////////////////////////////////////////////////////////


#include <Resources.h>
#include <Sound.h>
#include <MacMemory.h>
#include <LowMem.h>
#include "FMExtern.h"

Boolean IsFileMakerNative();


const ProcInfoType  
        kProcFMEntryProcPtrInfo = kThinkCStackBased
        | STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(FMExternCallPtr)));


static OSErr GetFSSpecFromRefNum(const short refnum, FSSpec* fss)
{
    OSErr err;
    FCBPBRec pb;
    Str32   filename;
    
    if(!fss) 
      return paramErr;
    
    pb.ioRefNum = refnum;
    pb.ioFCBIndx = 0;
    pb.ioVRefNum = 0;
    pb.ioNamePtr = filename;
    err = PBGetFCBInfoSync(&pb);
    if(!err)
        err = FSMakeFSSpec(pb.ioVRefNum,pb.ioFCBParID,pb.ioNamePtr,fss);
    
    return err;

}


static OSErr FMFLGetLoadParams(const FSSpec* inFile, UInt32* offset, UInt32* length)
{
    Handle              top = LMGetTopMapHndl();
    CFragResourceHandle cfrg = NULL;
    Boolean             close = false;
    short               resfile = -1, 
                        i,
                        membercount = 0,
                        cur = CurResFile();
    OSErr               err = noErr;
    
    resfile = FSpOpenResFile(inFile, fsRdPerm);
    close = LMGetTopMapHndl() != top;
    err = ResError();
    
    if(resfile != -1 && err == noErr)
    {
        UseResFile(resfile);
        cfrg = (CFragResourceHandle)Get1Resource(kCFragResourceType, kCFragResourceID);
        err = ResError();
        
        if(cfrg == NULL && err == noErr)
            err = resNotFound;
        
        if(err == noErr)
        {
            CFragResourcePtr    p = *cfrg;
            
            if(GetHandleSize((Handle)cfrg) >= sizeof(CFragResource))
            {
                membercount = p->memberCount;
                    
                if(membercount >= 1)
                {
                    CFragResourceMemberPtr  member = &p->firstMember;
                    
                    for(i=0; i<membercount; i++)
                    {
                        if(member->currentVersion == 1024)
                        {
                            *offset = member->offset;
                            *length = member->length;
                            break;
                        }
                    
                        member = NextCFragResourceMemberPtr(member);
                    }
                    
                    if(membercount == i)
                    {
                        err = cfragNoSectionErr;
                    }
                }
                else 
                    err = inputOutOfBounds;
            }
            else
                err = inputOutOfBounds;
        }
        
        if(cfrg)
            ReleaseResource((Handle)cfrg);
    }
    
    
    UseResFile(cur);
    
    if(close)
        CloseResFile(resfile);
    

    return err;
}


static OSErr FMFLGetFragment(UniversalProcPtr* outMain)
{
    FSSpec              fss;
    OSErr               err = noErr;
    CFragConnectionID   cid = NULL;
    Ptr                 entry = NULL;
    UniversalProcPtr    newEntry = NULL;
    UInt32              offset = 0;
    UInt32              length = 0;
    
    *outMain = NULL;
    
    err = GetFSSpecFromRefNum(CurResFile(), &fss);

    if(err == noErr)
        err = FMFLGetLoadParams(&fss, &offset, &length);
    
    if(err == noErr)
    {   
        err = GetDiskFragment(&fss, offset, length, NULL, kFindCFrag | kReferenceCFrag, &cid, NULL, NULL);
        if(err) err = GetDiskFragment(&fss, offset, length, NULL, kPrivateCFragCopy, &cid, NULL, NULL);
    }
    
    if(err == noErr)
        err = FindSymbol(cid, "\pFMExternCallProc", &entry, NULL);

    if(err == noErr)
    {
        newEntry = NewRoutineDescriptorTrap ((ProcPtr)entry, kProcFMEntryProcPtrInfo, kPowerPCISA);
    
        if(newEntry == NULL)
            err = memFullErr;
        else
            *outMain = newEntry;
    }
    
    if(err)
    {
        CloseConnection(&cid);
    }
    
    return err;
}


OSErr FMFLInit(FMExternCallPtr f)
{
    static int          first_entry = 1;
    OSErr               err = noErr;
    Handle              h = NULL;
    Size                s = 0;
    Boolean             detached = 0;
    UniversalProcPtr    upp = NULL;
    
    if(!first_entry) return noErr;
    
    first_entry = 0;
    
    if(IsFileMakerNative())
    {
        h = RecoverHandle((Ptr)(f->entryPoint));
        err = MemError();
        
        if(err == noErr)
        {
            s = GetHandleSize(h);
            err = MemError();
        }                   

        if(err == noErr)
        {
            detached = (HGetState(h) & 0x20) == 0;

            err =  FMFLGetFragment(&upp);
            
            
            if(err == noErr && upp != NULL)
            {
                BlockMoveData(upp, (Ptr)f->entryPoint, sizeof(RoutineDescriptor));
                
                ((void (*)(FMExternCallPtr))(f->entryPoint))(f);
                
                f->whichCall=100;
                f->parm2 = (long)h;
                ((void (*)(FMExternCallPtr))(f->entryPoint))(f);
                
                err = 1;
            }
        }
    }
    
    return err;
}