//////////////////////////////////////////////////////////////////////////////////////
//  
//  FMTemplate.c - Main entry point and underlying code for 24U FM Template
//
//  Modified public version distributed as part of 24U FM Template 1.1
//  
//  Copyright 2002-2004 by 24U Software. All rights reserved.
//
//  Written by Tomas Zahradnicky
//
////////////////////////////////////////////////////////////////////////////////
//
//  24U Sample Code - Terms of use
//
//  You may incorporate this sample code into your products without
//  restriction, though the code has been provided "AS IS" and the
//  responsibility for its operation is 100% yours.
//
//  You are not however permitted to redistribute the source
//  as "24U Sample Code" after having made changes.
//
//  If you are going to redistribute the source, we require that
//  you make it clear in the source that the code was descended from
//  24U Sample Code, but that you've made changes.
//
//  If you incorporate this sample code into your own product which you
//  distribute, you must include the following statement in the copyright
//  notices of your documentation: "Portions of the software are based
//  on 24U Sample Code which is copyright 2002-2004 by 24U Software.
//  All rights reserved."
//
////////////////////////////////////////////////////////////////////////

//
//  This source file was derived from FileMaker template plug-in provided
//  by FileMaker, Inc. as a part of FileMaker Pro 4.0 Developer Edition.
//
////////////////////////////////////////////////////////////////////////////////

//
//  Use this file as a part of all compiled parts of your plug-in.
//  Without this file your own code will never get called by FileMaker Pro as
//  it contains the key routines which connect the EFP API to your own code.
//
////////////////////////////////////////////////////////////////////////////////


#pragma mark Includes
//:::::::::::::::::::::::::::::   Includes   :::::::::::::::::::::::::::::://

#include "FMFlags.h"

#if TARGET_OS_MAC
 #if TARGET_CPU_68K
    #include <A4Stuff.h>
 #else
    #define EnterCodeResource()
    #define ExitCodeResource()
 #endif
#endif

#include "FMExtern.h"
#include "FMOptional.h"
#include "MacWinDefs.h"

#if __ide_target("68K")
    #include "FMFatLoader.h"
#endif


#pragma mark Forward defs
//:::::::::::::::::::::::::::   Forward defs   :::::::::::::::::::::::::::://

static long Do_Init(short version);
static void Do_Shutdown(void);
static void Do_Idle(FChar unsafeCalls);
static void Do_External(long functionId, FHandle parameter, FHandle result);
static void Do_AppPrefs(void);
static void Do_AppPrefs(void);
static void ShutSomethingDown(void);
static int  StartSomethingUp(void); 

#if __ide_target("68K")
    static short InitLoader(void);
#endif

enum {
    kSuccessLoading = 1,
    kFailureLoading = 0
};

///////////////////////////////////
typedef void (*ExtFuncPtr)(FHandle parameter, FHandle result);

        ExtFuncPtr gJumpTable[10] = {0,};
        int		   gStartedUp = 0;

///////////////////////////////////

#pragma mark Main Entry Point
//:::::::::::::::::::::::::   Main Entry Point   :::::::::::::::::::::::::://
// This is the entry point into a FileMaker plugin. You should not have to //
// modify anything in this section.                                        //

#ifdef __cplusplus
extern "C"
 {
#endif

// 68K entry point definition
#if FM_CPU_68K
    pascal void main(FMExternCallPtr pb);
    pascal void main(FMExternCallPtr pb)
#else

// PowerPC entry point definition
#if FM_CPU_PPC

 #if TARGET_API_MAC_CARBON
    void __declspec(export) FMExternCarbonCallProc(FMExternCallPtr pb);
    void __declspec(export) FMExternCarbonCallProc(FMExternCallPtr pb)
 #else
    void __declspec(export) FMExternCallProc(FMExternCallPtr pb);
    void __declspec(export) FMExternCallProc(FMExternCallPtr pb)
 #endif 
    
// Windows entry point definition
#else
    void __declspec(dllexport) FMExternCallProc(FMExternCallPtr pb);
    void __declspec(dllexport) FMExternCallProc(FMExternCallPtr pb)

#endif
#endif
{

    #if __ide_target("PPC Classic")
        static Handle sFMXTHandle = NULL;
    #endif
    
    // Setup code
    #if !FM_CPU_X86
        short fmOriginalResPath = CurResFile();
        EnterCodeResource();
        UseResFile(pb->resourceID);
    #endif
    
    gFMExternCallPtr = pb;
    
    #if __ide_target("68K")
        if (InitLoader() == 1)
        {
            // PowerPC fragment loaded successfully, stop using 68k code
            goto stop_using_68k_code_now;
        }
    #endif

    #if __ide_target("PPC Classic")
        if (sFMXTHandle)
        {
            HUnlock(sFMXTHandle);
            SetHandleSize(sFMXTHandle, sizeof(RoutineDescriptor));
            HLock(sFMXTHandle);
            sFMXTHandle = NULL;
        }
    #endif
    
    // Message dispatcher
        switch (pb->whichCall) {
            case kFMXT_Init:
                pb->result = Do_Init(pb->extnVersion);
                break;
            case kFMXT_Idle:
                Do_Idle(pb->unsafeCalls);
                break;
            case kFMXT_External:
                Do_External(pb->parm2, (FHandle)(pb->parm3), (FHandle)(pb->result));
                break;
            case kFMXT_Shutdown:
                Do_Shutdown();
                break;
            case kFMXT_DoAppPreferences:
                Do_AppPrefs();
                break;
            #if __ide_target("PPC Classic")
                case 100:
                    sFMXTHandle = (Handle)pb->parm2;
                    break;
            #endif
        }

    #if __ide_target("68K")
        stop_using_68k_code_now:
    #endif
    
    // Exit code
    #if !FM_CPU_X86
        UseResFile(fmOriginalResPath);
        ExitCodeResource();
    #endif
    
} // FMExternCallProc

#ifdef __cplusplus
}
#endif


#pragma mark -

//:::::::::::::::::::::::::   Function Routines   ::::::::::::::::::::::::://
// These are three simple external plugin functions that FileMaker calcs   //
// can call when this plugin is enabled. The input and result parms are    //
// Mac style handles that contain text. This means that numbers are also   //
// represented as text. The result handle is always empty when these       //
// routines are called. Only manipulate the input and result parms using   //
// the memory manager callback routines that are defined in FMExtern.h.    //
// On the Mac they will be real Macintosh handles, but you cannot depend   //
// on which heap they may be located in. On the PC side this handles exist //
// in FileMaker's memory manager pool and has no relation to anything      //
// called a HANDLE in the Win32 API's.                                     //


#pragma mark -

//::::::::::::::::::::::::   Something Routines   ::::::::::::::::::::::::://
// These are just a set of dummy routines that represent some service that //
// has to be initialize successfully before being used, needs idle time to //
// do so kind of processing/polling, and needs to be shutdown cleanly. If  //
// you are just implementing some simple external functions that do not    //
// require any type of system service then you can stop the messages that  //
// trigging these by changing the flags in the option string.              //

static void ShutSomethingDown(void)
{
    Shutdown();
}


static int StartSomethingUp(void) 
{
    gStartedUp = Startup();
    
    return gStartedUp;
}


#pragma mark -
//::::::::::::::::::::::::  Init/Shutdown Handler  :::::::::::::::::::::::://
// These routines are called when the plugin is enabled or disable in      //
// FileMaker. The user can enable or disable any plugin at any time by     //
// going to the application prefences panel and changing the state of the  //
// check box in front of the plugin name. The plugin is enabled or         //
// disable immediately after the click. All plugins that are disabled are  //
// rememeber in the FileMaker preferences file. During launch FileMaker    //
// will enable all plugins it can find except for the onces explicitly     //
// disabled by the user.                                                   //

static void InitJT()
{
    #ifdef FUNCTION_1_NAME
      extern void C_FUNCTION_1_NAME(FHandle,FHandle);
      #ifdef FUNCTION_2_NAME
        extern void C_FUNCTION_2_NAME(FHandle,FHandle);
        #ifdef FUNCTION_3_NAME
          extern void C_FUNCTION_3_NAME(FHandle,FHandle);
          #ifdef FUNCTION_4_NAME
            extern void C_FUNCTION_4_NAME(FHandle,FHandle);
            #ifdef FUNCTION_5_NAME
              extern void C_FUNCTION_5_NAME(FHandle,FHandle);
              #ifdef FUNCTION_6_NAME
                extern void C_FUNCTION_6_NAME(FHandle,FHandle);
                #ifdef FUNCTION_7_NAME
                  extern void C_FUNCTION_7_NAME(FHandle,FHandle);
                  #ifdef FUNCTION_8_NAME
                    extern void C_FUNCTION_8_NAME(FHandle,FHandle);
                    #ifdef FUNCTION_9_NAME
                      extern void C_FUNCTION_9_NAME(FHandle,FHandle);
                      gJumpTable[8] = C_FUNCTION_9_NAME;
                    #endif
                    gJumpTable[7] = C_FUNCTION_8_NAME;
                  #endif
                  gJumpTable[6] = C_FUNCTION_7_NAME;
                #endif
                gJumpTable[5] = C_FUNCTION_6_NAME;
              #endif
              gJumpTable[4] = C_FUNCTION_5_NAME;
            #endif
            gJumpTable[3] = C_FUNCTION_4_NAME;
          #endif
          gJumpTable[2] = C_FUNCTION_3_NAME;
        #endif
        gJumpTable[1] = C_FUNCTION_2_NAME;
      #endif
      gJumpTable[0] = C_FUNCTION_1_NAME;
    #else
     #error You must define at least one function!! Look into MacWinDefs.h
    #endif
}

static long Do_Init(short version) {

    int somethingResult;


// Make sure this is a version of FileMaker I can talk to
    if ((version < kMinExtnVersion) || (version > kMaxExtnVersion)) {
        // Do not display a dialog at this point, let
        // FileMaker do the error handling instead.
        return kBadExtnVersion;
    }

// There may be something you have to start up here, and maybe it can fail
    somethingResult = StartSomethingUp();
    if (somethingResult != kSuccessLoading) {
        return kDoNotEnable;
    }

    InitJT();
    
// Everything is fine
    return kCurrentExtnVersion;

}

static void Do_Shutdown(void) {

	gStartedUp = 0;
    ShutSomethingDown();

}

//:::::::::::::::::::::::::::::  Idle Handler  :::::::::::::::::::::::::::://
// This is an optional routine. It is only called if the ninth character   //
// in the option string is the character Y. There are two points in File-  //
// Maker where this called. If the "unsafeCalls" parameter is non-zero     //
// then the routine has been called at the same time the low-level network //
// code has been called in FileMaker. Do not do any long processing, UI,   //
// or event munging when unsafeCalls is non-zero. The other time when      //
// FileMaker will call the idle handler is when FileMaker detects free     //
// time and does it's own internal idle handling.                          //

static void Do_Idle(FChar unsafeCalls) {

// We are going to pretend that the "Something" code needs to make some high level calls
    if (!unsafeCalls) {
        if (gStartedUp) {
            Idle();
        }
    }

}

//::::::::::::::::::::::  External Function Handler  :::::::::::::::::::::://
// This is an optional routine. It is only called if the seventh character //
// in the option string is the character Y. The name of all the external   //
// functions you want to export are listed in a resource. On the Mac they  //
// are the in the 'STR#' id 144 resource and on the PC they start at       //
// string number 144. The first name will be mapped to functionID 0 and    //
// incremented by one for each following name. See the section on Funtion  //
// Routines so see how to implement an external function for FileMaker.    //



static void Do_External(long functionId, FHandle parameter, FHandle result) 
{
    ExtFuncPtr  externalFunction;

    if(functionId < 10)
    {
        externalFunction = gJumpTable[functionId];
        if(externalFunction)
            externalFunction(parameter,result);
    }
} // Do_External

//::::::::::::::::::::::::::  Preference Handler  ::::::::::::::::::::::::://
// This is an optional routine. It is only called if the sixth character   //
// in the option string is the character Y, also a config button will be   //
// enabled in the Plug-In preference panel. When the button is press this  //
// handler will be called and it is expected that you are going to put up  //
// a dialog box of some sort. An options you want to save should be put in //
// your own preference file on the Mac and your own registery entry or     //
// ini-file on the PC.                                                     //

static void Do_AppPrefs(void) 
{
    Preferences();  
}

#pragma mark -

#if __ide_target("68K")
    static short InitLoader(void)
    {
        return FMFLInit(gFMExternCallPtr);
    }
#endif