//////////////////////////////////////////////////////////////////////////////////////
//  
//  ParamParser.cp - Useful utility functions for parsing multiple input parameters
//
//  Modified public version distributed as part of 24U FM Template 1.1
//  
//  Copyright 2002-2004 by 24U Software. All rights reserved.
//
//  Written by Tomas Zahradnicky
//
////////////////////////////////////////////////////////////////////////////////
//
//  24U Sample Code - Terms of use
//
//  You may incorporate this sample code into your products without
//  restriction, though the code has been provided "AS IS" and the
//  responsibility for its operation is 100% yours.
//
//  You are not however permitted to redistribute the source
//  as "24U Sample Code" after having made changes.
//
//  If you are going to redistribute the source, we require that
//  you make it clear in the source that the code was descended from
//  24U Sample Code, but that you've made changes.
//
//  If you incorporate this sample code into your own product which you
//  distribute, you must include the following statement in the copyright
//  notices of your documentation: "Portions of the software are based
//  on 24U Sample Code which is copyright 2002-2004 by 24U Software.
//  All rights reserved."
//
////////////////////////////////////////////////////////////////////////

//
//  Use this file as a part of all compiled parts of your plug-in.
//  It contains several handy functions for manipulating input parameters
//  in your external functions.
//
////////////////////////////////////////////////////////////////////////////////

#include <vector>
#include <string>
#include <stdio.h>
#include <string.h>

extern "C" {
#include "FMExtern.h"
}

#include "ParamParser.h"

static void PPDeleteString(std::string* iString)
{
    delete iString;
}

static std::vector<std::string*>* PPTokenizePointer(FPtr iParam, int iParamSize, char iDelimChar)
{
    int i;
    std::string str((const char*)iParam,iParamSize);
    std::vector<std::string*>* vec = new std::vector<std::string*>; 
    std::string* indpar = NULL;
    
    if( vec && (str.length()) )
    {
        for(i=0;i<str.length();i++)
        {
            if( str[i] == iDelimChar )
            {
                indpar = new std::string;
                
                if( indpar )
                {
                    indpar->assign(str,0,i);
                    
                    vec->push_back(indpar);
                    
                    str.erase(0,i+1);
                    
                    i = -1;
                }
                else
                {
                    std::for_each(vec->begin(),vec->end(),PPDeleteString);
                    delete vec;
                    vec = NULL;
                    break;
                }
            }
        } 

        if( vec && (str.length()>=0) )
        {
            indpar = new std::string;
            
            if( indpar )
            {
                indpar->assign(str,0,i);
                
                vec->push_back(indpar);
            }
            else
            {
                std::for_each(vec->begin(),vec->end(),PPDeleteString);
                delete vec;
                vec = NULL;
            }
        }    
    }

    return vec;
}

static std::vector<std::string*>* PPTokenizeHandle(FHandle iParam, char iDelimChar)
{
    std::vector<std::string*>*  result = NULL;
    
    FMX_LockHandle(iParam);    

        result = PPTokenizePointer(*iParam,FMX_GetHandleSize(iParam),iDelimChar);

    FMX_UnlockHandle(iParam);    
    
    return result;
}

#pragma mark -

short CountParams(FHandle iParam)
{
    int                         count = 0;
    std::vector<std::string*>*  par = NULL;
     
    if (iParam)
        par = PPTokenizeHandle(iParam,'|');

    if( par )
    {
        count = par->size();
        std::for_each(par->begin(),par->end(),PPDeleteString);
        delete par;
    }
    
    return count;
}
    
FPtr GetIndParam(FHandle iParam, short ind)
{
    FPtr                        result = NULL;
    std::vector<std::string*>*  par = NULL;
     
    --ind;
    
    if (iParam)
        par = PPTokenizeHandle(iParam,'|');

    if( par )
    {
        if( ind < par->size() )
        {
            result = FMX_NewPointer(0[par][ind]->size()+1);
            
            if( result )
                strcpy((char*)result, 0[par][ind]->c_str());
        }
        
        std::for_each(par->begin(),par->end(),PPDeleteString);
        delete par;
    }
    
    return result;
}

FPtr GetParamsFromIndToEnd(FHandle iParam, short ind)
{
    int                         i;
    FPtr                        result = NULL;
    std::vector<std::string*>*  par = NULL;
    std::string                 concat;
    
    --ind;
    
    if (iParam)
        par = PPTokenizeHandle(iParam,'|');

    if( par )
    {
        if( ind < par->size() )
        {
            for(i=ind; i<par->size(); ++i)
            {
                if(i>ind) concat.append("|");
                concat.append(0[par][i][0]);
            }
                
            result = FMX_NewPointer(concat.size()+1);
            
            if( result )
                strcpy((char*)result, concat.c_str());
        }
        
        std::for_each(par->begin(),par->end(),PPDeleteString);
        delete par;
    }
    
    return result;
}
