//////////////////////////////////////////////////////////////////////////////////////
//  
//  ParamParser.h - Header file for 24U ParamParser utility functions set
//
//  Modified public version distributed as part of 24U FM Template 1.0
//  
//  Copyright 2002-2003 by 24U Software. All rights reserved.
//
//  Written by Tomas Zahradnicky & HOnza Koudelka
//
////////////////////////////////////////////////////////////////////////////////
//
//  24U Sample Code - Terms of use
//
//  You may incorporate this sample code into your products without
//  restriction, though the code has been provided "AS IS" and the
//  responsibility for its operation is 100% yours.
//
//  You are not however permitted to redistribute the source
//  as "24U Sample Code" after having made changes.
//
//  If you are going to redistribute the source, we require that
//  you make it clear in the source that the code was descended from
//  24U Sample Code, but that you've made changes.
//
//  If you incorporate this sample code into your own product which you
//  distribute, you must include the following statement in the copyright
//  notices of your documentation: "Portions of the software are based
//  on 24U Sample Code which is copyright 2002-2004 by 24U Software.
//  All rights reserved."
//
////////////////////////////////////////////////////////////////////////

//
//  Use this file as a part of all compiled parts of your plug-in.
//  It contains several handy functions for manipulating input parameters
//  in your external functions.
//
////////////////////////////////////////////////////////////////////////////////


#if __cplusplus
    extern  "C" {
#endif


#pragma mark CountParams

////////////////////////////////////////////////////////////////////////////////
//
    short CountParams(FHandle parameter);
//
////////////////////////////////////////////////////////////////////////////////
//
//  Assuming the parameter FHandle obtained from FileMaker Pro contains a list
//  of parameters delimited by the pipe character ('|'), this function counts
//  the number of parameters in the parameter FHandle.
//
//  parameter   <- the input parameter FHandle obtained from FileMaker Pro
//
//  result      -> number of individual parameters delimited by '|'
//
////////////////////////////////////////////////////////////////////////////////


#pragma mark GetIndParam

////////////////////////////////////////////////////////////////////////////////
//
    FPtr    GetIndParam(FHandle parameter, short ind);
//
////////////////////////////////////////////////////////////////////////////////
//
//  This function returns a pointer to a newly allocated buffer containing
//  a copy of a specific pipe-delimited parameter. The parameter is extended
//  with a null character to constitute a null-terminated string ("C string").
//
//  parameter   <- the input parameter FHandle obtained from FileMaker Pro
//
//  ind         <- numeric (1-based) index of the desired parameter
//
//  result      -> pointer to a new copy of the desired parameter from the
//                 parameter FHandle, zero if there is no parameter or not
//                 enough memory to allocate space for copying the parameter
//
//  IMPORTANT:     You are responsible for disposing of the allocated memory
//                 with the FMX_DisposePointer function when you are finished
//                 working with the string returned from this function.
//
////////////////////////////////////////////////////////////////////////////////


#pragma mark GetParamsFromIndToEnd

////////////////////////////////////////////////////////////////////////////////
//
    FPtr    GetParamsFromIndToEnd(FHandle parameter, short ind);
//
////////////////////////////////////////////////////////////////////////////////
//
//  This function returns a pointer to a newly allocated buffer containing
//  all pipe-delimited parameters starting with a specific one. This is
//  especially useful if you have a fixed number of parameters and want to
//  allow the last one to contain also the pipe character(s). A null character
//  is appended to constitute a null-terminated string ("C string").
//
//  parameter   <- the input parameter FHandle obtained from FileMaker Pro
//
//  ind         <- numeric (1-based) index of the first desired parameter
//
//  result      -> pointer to a new copy of the portion of the parameter
//                 FHandle starting with the desired parameter and continuing
//                 to the end of the parameter FHandle
//                 returns zero if there is no parameter or not enough memory
//                 to allocate space for copying the data
//
//  IMPORTANT:     You are responsible for disposing of the allocated memory
//                 with the FMX_DisposePointer function when you are finished
//                 working with the string returned from this function.
//
////////////////////////////////////////////////////////////////////////////////


#if __cplusplus
    }
#endif
