//////////////////////////////////////////////////////////////////////////////////////
//  
//  FMOptional.c - Functions for processing optional plug-in events
//
//  Modified public version distributed as part of 24U FM Template 1.1
//  
//  Copyright 2002-2004 by 24U Software. All rights reserved.
//
//  Written by Tomas Zahradnicky
//
////////////////////////////////////////////////////////////////////////////////
//
//  24U Sample Code - Terms of use
//
//  You may incorporate this sample code into your products without
//  restriction, though the code has been provided "AS IS" and the
//  responsibility for its operation is 100% yours.
//
//  You are not however permitted to redistribute the source
//  as "24U Sample Code" after having made changes.
//
//  If you are going to redistribute the source, we require that
//  you make it clear in the source that the code was descended from
//  24U Sample Code, but that you've made changes.
//
//  If you incorporate this sample code into your own product which you
//  distribute, you must include the following statement in the copyright
//  notices of your documentation: "Portions of the software are based
//  on 24U Sample Code which is copyright 2002-2004 by 24U Software.
//  All rights reserved."
//
////////////////////////////////////////////////////////////////////////

//
//  This source file contains code for the Startup, Shutdown, Preferences, and
//  Idle functions. Please see comments attached to each individual function
//  for instructions on writing code for them. We suggest that you don't add
//  more functions to this file if they would be referenced from other files.
//  If you have a need for utility functions that you would call from different
//  places, you should create a separate source file for them.
//
////////////////////////////////////////////////////////////////////////////////


#include "MacWinDefs.h"
#include "FMExtern.h"
#include "FMOptional.h"


////////////////////////////////////////////////////////////////////////////////
//
//  Startup()
//
/////////////////////////////////////////////////////////////////////////////
//
//  This function will get called once each time your plug-in needs to be
//  initialized. That is during the FileMaker Pro application startup, or
//  right after clicking on the checkbox that enables your plug-in in the
//  application's Preferences dialog.
//
//  If your plug-in has any software or hardware requirements that might
//  not be met, you should perform the necessary environment check here,
//  and disable the plug-in if the environment is not sufficient.
//
//  Return 1 as a result from this function to enable the plug-in. If you
//  decide that the plug-in should be disabled, return 0.
//  

char Startup(void)
{
    char enable = 1;    // Enable the plug-in by default
    
    // Write your initialization/environment check code here.
    // Set the enable variable to 0 if you want to disable the plug-in.
    
    return enable;      // Return 1 to enable the plug-in, or 0 to disable it
}


////////////////////////////////////////////////////////////////////////////////
//
//  Shutdown()
//
/////////////////////////////////////////////////////////////////////////////
//
//  When user disables your plug-in in the application's Preferences dialog
//  or when he quits FileMaker Pro, this will be the last function that will
//  get called.
//  
//  If your plug-in allocates any system resources, such as memory, that
//  remain allocated across multiple calls to the plug-in's functions, you
//  should release the resources and perform any necessary cleanup in this
//  function.
//
//  Keep in mind that once deactivated, your plug-in can be re-activated
//  in the application's Preferences dialog, so make sure you recover the
//  plug-in into exactly the same state it was before getting activated for
//  the first time.
//
//  This function does not take any input parameters nor does it have to
//  return any value as a result.
//

void Shutdown(void)
{
    // Write your shutdown/cleanup code here.
    // Don't forget to release any occupied system resources.
}


////////////////////////////////////////////////////////////////////////////////
//
//  Preferences()
//
/////////////////////////////////////////////////////////////////////////////
//
//  If you set the WANT_PREFS constant in the MacWinDefs.h file to 1, your
//  plug-in will have an enabled "Configure..." button in the Plug-In pane
//  of the application's Preferences dialog. When a user clicks on that
//  button, the following function will get called. After then, you are
//  supposed to display your plug-in's preferences dialog to let user modify
//  your plug-in's preferences.
//
//  If you don't want to use any user-configurable values in your plug-in,
//  you should set the WANT_PREFS constant to 0. Then the following function
//  should never get called. Anyway, it is still a good idea to at least
//  beep or display an error message in here, just for the case you forget
//  to set the WANT_PREFS constant to 0.
//
//  This function does not take any input parameters nor does it have to
//  return any value as a result.
//

void Preferences(void)
{
    // Write your preferences dialog handling code here.
    // Don't forget to set the WANT_PREFS constant in the MacWinDefs.h
    // accordingly.
    
    #if TARGET_OS_MAC
        NoteAlert(23000, 0);    // On a Mac, display the alert ID 23000
                                // defined in the MyPlugin.rsrc file
    #else
        MessageBox(         	// On Windows, use standard message box
			NULL, 
			"This plug-in does not have any configurable options",
			"24U Template Plug-in Preferences",
			MB_OK
		);
    #endif
}


////////////////////////////////////////////////////////////////////////////////
//
//  Idle()
//
/////////////////////////////////////////////////////////////////////////////
//
//  If the WANT_IDLE constant is set to 1 in the MacWinDefs.h file, and if
//  your plug-in is enabled, this function will get called periodically when
//  the FileMaker Pro application has nothing else to do. There is no control
//  over how often this will happen, so you should not rely on any specific
//  frequency.
//
//  You can put any idle processing in this function, such as background
//  activity, scheduled actions, etc., but you should return the control
//  to FileMaker Pro quickly as it is stuck while you are doing your idle
//  processing.
//
//  This function does not take any input parameters nor does it have to
//  return any value as a result.
//

void Idle(void) 
{
    // Write your idle processing code here.
    // Keep the code short and quick so that you don't steal too much
    // processor time from FileMaker Pro.
    
    // Don't forget to set the WANT_IDLE constant in the MacWinDefs.h
    // file to 1 if you want this function to get called.
}
