//////////////////////////////////////////////////////////////////////////////////////
//  
//  PluginVersion.c - Default external function for returning plug-in version
//
//  Modified public version distributed as part of 24U FM Template 1.1
//  
//  Copyright 2002-2004 by 24U Software. All rights reserved.
//
//  Written by Tomas Zahradnicky & HOnza Koudelka
//
////////////////////////////////////////////////////////////////////////////////
//
//  24U Sample Code - Terms of use
//
//  You may incorporate this sample code into your products without
//  restriction, though the code has been provided "AS IS" and the
//  responsibility for its operation is 100% yours.
//
//  You are not however permitted to redistribute the source
//  as "24U Sample Code" after having made changes.
//
//  If you are going to redistribute the source, we require that
//  you make it clear in the source that the code was descended from
//  24U Sample Code, but that you've made changes.
//
//  If you incorporate this sample code into your own product which you
//  distribute, you must include the following statement in the copyright
//  notices of your documentation: "Portions of the software are based
//  on 24U Sample Code which is copyright 2002-2004 by 24U Software.
//  All rights reserved."
//
////////////////////////////////////////////////////////////////////////

//
//  This source file contains code for the PluginVersion function. Since it is
//  recommended for every FileMaker Plug-in to have a function for checking
//  the plug-in's version, we have written a simple version function for you.
//  When an empty string is passed to this function, it simply returns the
//  version of this plug-in, as defined in the MacWinDefs.h header file.
//
//  As a bonus for you, if you pass the string "platform" as a parameter to this
//  function, it will respond with the information about which code is being
//  used, whether it is Mac 68K code, Mac PPC Classic code, Mac PPC Carbon code,
//  or Windows code. This bonus functionality is especially useful for testing
//  if the Fat Loader is working as expected.
//
////////////////////////////////////////////////////////////////////////////////


#include "MacWinDefs.h"
#include "FMExtern.h"
#include "ParamParser.h"

void PluginVersion(FHandle in, FHandle out);


void PluginVersion(FHandle in, FHandle out)
{   
    long    insize = FMX_GetHandleSize(in);         // Check the size of input parameter
    long    outsize = 0;                            // Default size of result is zero
    FPtr    param = NULL;                           // Avoid using invalid pointer
    char*   result = PLUGIN_VERSION;                // Return plug-in version by default
    
    if (insize > 0)                                 // Was anything passed in?
    {
        param = GetIndParam(in, 1);                 // Get the first parameter as CString
        if (param)
            if (!strcmp((char*)param, "platform"))  // Is the parameter "platform"?
            {
                result =                            // Set the result according to the
                #if FM_CPU_68K                      // current platform
                    "Mac 68K"
                #elif FM_CPU_X86
                    "Windows"
                #elif FM_CPU_PPC
                    #if TARGET_API_MAC_CARBON
                        "Mac PPC Carbon"
                    #else
                        "Mac PPC Classic"
                    #endif
                #else
                    "Unknown"
                #endif
                ;
            }
    }
    
    outsize = strlen(result);                       // Calculate result size
    
    FMX_SetHandleSize(out, outsize);                // Allocate memory for result
    
    if(FMX_MemoryError() == 0)                      // Was the memory allocated?
    {
        memcpy((*out), result, outsize);            // Copy our string to the buffer
    }
    
    if (param) FMX_DisposePointer(param);           // Release memory occupied by parameter
}
